@@ -35,6 +35,10 @@
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#include <time.h>
 
 #ifdef HAVE_LANGINFO_YESEXPR
 #include <langinfo.h>
@@ -99,8 +103,16 @@ event_t mutt_getch (void)
     mutt_query_exit ();
 
   if(ch == ERR)
+  {
+    /* either timeout or the terminal has been lost */
+    if (!isatty (0))
+    {
+      endwin ();
+      exit (1);
+    }
     return err;
-  
+  }
+
   if ((ch & 0x80) && option (OPTMETAKEY))
   {
     /* send ALT-x as ESC-x */
@@ -292,7 +304,7 @@ void mutt_curses_error (const char *fmt, ...)
   
   dprint (1, (debugfile, "%s\n", scratch));
   mutt_format_string (Errorbuf, sizeof (Errorbuf),
-		      0, COLS-2, 0, 0, scratch, sizeof (scratch), 0);
+		      0, COLS-2, FMT_LEFT, 0, scratch, sizeof (scratch), 0);
 
   if (!option (OPTKEEPQUIET))
   {
@@ -317,7 +329,7 @@ void mutt_curses_message (const char *fmt, ...)
   va_end (ap);
 
   mutt_format_string (Errorbuf, sizeof (Errorbuf),
-		      0, COLS-2, 0, 0, scratch, sizeof (scratch), 0);
+		      0, COLS-2, FMT_LEFT, 0, scratch, sizeof (scratch), 0);
 
   if (!option (OPTKEEPQUIET))
   {
@@ -335,6 +347,8 @@ void mutt_progress_init (progress_t* progress, const char *msg,
 			 unsigned short flags, unsigned short inc,
 			 long size)
 {
+  struct timeval tv = { 0, 0 };
+
   if (!progress)
     return;
   memset (progress, 0, sizeof (progress_t));
@@ -342,58 +356,88 @@ void mutt_progress_init (progress_t* progress, const char *msg,
   progress->flags = flags;
   progress->msg = msg;
   progress->size = size;
-  mutt_progress_update (progress, 0);
+  if (progress->size) {
+    if (progress->flags & M_PROGRESS_SIZE)
+      mutt_pretty_size (progress->sizestr, sizeof (progress->sizestr),
+			progress->size);
+    else
+      snprintf (progress->sizestr, sizeof (progress->sizestr), "%ld",
+		progress->size);
+  }
+  if (!inc)
+  {
+    if (size)
+      mutt_message ("%s (%s)", msg, progress->sizestr);
+    else
+      mutt_message (msg);
+    return;
+  }
+  if (gettimeofday (&tv, NULL) < 0)
+    dprint (1, (debugfile, "gettimeofday failed: %d\n", errno));
+  /* if timestamp is 0 no time-based suppression is done */
+  if (TimeInc)
+    progress->timestamp = tv.tv_sec * 1000 + tv.tv_usec / 1000;
+  mutt_progress_update (progress, 0, 0);
 }
 
-void mutt_progress_update (progress_t* progress, long pos)
+void mutt_progress_update (progress_t* progress, long pos, int percent)
 {
   char posstr[SHORT_STRING];
   short update = 0;
-
-  if (!pos)
-  {
-    if (!progress->inc)
-      mutt_message (progress->msg);
-    else
-    {
-      if (progress->size)
-      {
-	if (progress->flags & M_PROGRESS_SIZE)
-	  mutt_pretty_size (progress->sizestr, sizeof (progress->sizestr), progress->size);
-	else
-	  snprintf (progress->sizestr, sizeof (progress->sizestr), "%ld", progress->size);
-      }
-      progress->pos = 0;
-    }
-  }
+  struct timeval tv = { 0, 0 };
+  unsigned int now = 0;
 
   if (!progress->inc)
-    return;
+    goto out;
 
-  if (progress->flags & M_PROGRESS_SIZE)
-  {
-    if (pos >= progress->pos + (progress->inc << 10))
-    {
-      pos = pos / (progress->inc << 10) * (progress->inc << 10);
-      mutt_pretty_size (posstr, sizeof (posstr), pos);
-      update = 1;
-    }
-  }
-  else if (pos >= progress->pos + progress->inc)
-  {
-    snprintf (posstr, sizeof (posstr), "%ld", pos);
+  /* refresh if size > inc */
+  if (progress->flags & M_PROGRESS_SIZE &&
+      (pos >= progress->pos + (progress->inc << 10)))
     update = 1;
+  else if (pos >= progress->pos + progress->inc)
+    update = 1;
+
+  /* skip refresh if not enough time has passed */
+  if (update && progress->timestamp && !gettimeofday (&tv, NULL)) {
+    now = tv.tv_sec * 1000 + tv.tv_usec / 1000;
+    if (now && now - progress->timestamp < TimeInc)
+      update = 0;
   }
 
+  /* always show the first update */
+  if (!pos)
+    update = 1;
+
   if (update)
   {
-    progress->pos = pos;
-    if (progress->size)
-      mutt_message ("%s %s/%s", progress->msg, posstr, progress->sizestr);
+    dprint (1, (debugfile, "Updating progress: %ld\n", pos));
+    if (progress->flags & M_PROGRESS_SIZE)
+    {
+      pos = pos / (progress->inc << 10) * (progress->inc << 10);
+      mutt_pretty_size (posstr, sizeof (posstr), pos);
+    }
     else
-      mutt_message ("%s %s", progress->msg, posstr);
+      snprintf (posstr, sizeof (posstr), "%ld", pos);
+    
+    progress->pos = pos;
+    if (now)
+      progress->timestamp = now;
+
+    if (progress->size > 0)
+    {
+      mutt_message ("%s %s/%s (%d%%)", progress->msg, posstr, progress->sizestr,
+		    percent > 0 ? percent : progress->pos * 100 / progress->size);
+    }
+    else
+    {
+      if (percent > 0)
+	mutt_message ("%s %s (%d%%)", progress->msg, posstr, percent);
+      else
+	mutt_message ("%s %s", progress->msg, posstr);
+    }
   }
 
+out:
   if (pos >= progress->size)
     mutt_clear_error ();
 }
@@ -411,6 +455,8 @@ void mutt_show_error (void)
 
 void mutt_endwin (const char *msg)
 {
+  int e = errno;
+
   if (!option (OPTNOCURSES))
   {
     CLEARLINE (LINES - 1);
@@ -425,6 +471,8 @@ void mutt_endwin (const char *msg)
     puts (msg);
     fflush (stdout);
   }
+
+  errno = e;
 }
 
 void mutt_perror (const char *s)
@@ -641,7 +689,7 @@ int mutt_addwch (wchar_t wc)
 
 void mutt_format_string (char *dest, size_t destlen,
 			 int min_width, int max_width,
-			 int right_justify, char m_pad_char,
+			 int justify, char m_pad_char,
 			 const char *s, size_t n,
 			 int arboreal)
 {
@@ -688,7 +736,7 @@ void mutt_format_string (char *dest, size_t destlen,
   w = (int)destlen < min_width ? destlen : min_width;
   if (w <= 0)
     *p = '\0';
-  else if (right_justify)
+  else if (justify == FMT_RIGHT)	/* right justify */
   {
     p[w] = '\0';
     while (--p >= dest)
@@ -696,7 +744,27 @@ void mutt_format_string (char *dest, size_t destlen,
     while (--w >= 0)
       dest[w] = m_pad_char;
   }
-  else
+  else if (justify == FMT_CENTER)	/* center */
+  {
+    char *savedp = p;
+    int half = (w+1) / 2; /* half of cushion space */
+
+    p[w] = '\0';
+
+    /* move str to center of buffer */
+    while (--p >= dest)
+      p[half] = *p;
+
+    /* fill rhs */
+    p = savedp + half;
+    while (--w >= half)
+      *p++ = m_pad_char;
+
+    /* fill lhs */
+    while (half--)
+      dest[half] = m_pad_char;
+  }
+  else					/* left justify */
   {
     while (--w >= 0)
       *p++ = m_pad_char;
@@ -718,13 +786,15 @@ static void mutt_format_s_x (char *dest,
 			     const char *s,
 			     int arboreal)
 {
-  int right_justify = 1;
+  int justify = FMT_RIGHT;
   char *p;
   int min_width;
   int max_width = INT_MAX;
 
   if (*prefix == '-')
-    ++prefix, right_justify = 0;
+    ++prefix, justify = FMT_LEFT;
+  else if (*prefix == '=')
+    ++prefix, justify = FMT_CENTER;
   min_width = strtol (prefix, &p, 10);
   if (*p == '.')
   {
@@ -735,7 +805,7 @@ static void mutt_format_s_x (char *dest,
   }
 
   mutt_format_string (dest, destlen, min_width, max_width,
-		      right_justify, ' ', s, mutt_strlen (s), arboreal);
+		      justify, ' ', s, mutt_strlen (s), arboreal);
 }
 
 void mutt_format_s (char *dest,
@@ -756,7 +826,7 @@ void mutt_format_s_tree (char *dest,
 
 /*
  * mutt_paddstr (n, s) is almost equivalent to
- * mutt_format_string (bigbuf, big, n, n, 0, ' ', s, big, 0), addstr (bigbuf)
+ * mutt_format_string (bigbuf, big, n, n, FMT_LEFT, ' ', s, big, 0), addstr (bigbuf)
  */
 
 void mutt_paddstr (int n, const char *s)
@@ -792,6 +862,62 @@ void mutt_paddstr (int n, const char *s)
     addch (' ');
 }
 
+/* See how many bytes to copy from string so its at most maxlen bytes
+ * long and maxwid columns wide */
+int mutt_wstr_trunc (const char *src, size_t maxlen, size_t maxwid, size_t *width)
+{
+  wchar_t wc;
+  int w = 0, l = 0, cl;
+  size_t cw, n;
+  mbstate_t mbstate;
+
+  if (!src)
+    goto out;
+
+  n = mutt_strlen (src);
+
+  memset (&mbstate, 0, sizeof (mbstate));
+  for (w = 0; n && (cl = mbrtowc (&wc, src, n, &mbstate)); src += cl, n -= cl)
+  {
+    if (cl == (size_t)(-1) || cl == (size_t)(-2))
+      cw = cl = 1;
+    else
+      cw = wcwidth (wc);
+    if (cl + l > maxlen || cw + w > maxwid)
+      break;
+    l += cl;
+    w += cw;
+  }
+out:
+  if (width)
+    *width = w;
+  return l;
+}
+
+/*
+ * returns the number of bytes the first (multibyte) character
+ * of input consumes:
+ * 	< 0 ... conversion error
+ * 	= 0 ... end of input
+ * 	> 0 ... length (bytes)
+ */
+int mutt_charlen (const char *s, int *width)
+{
+  wchar_t wc;
+  mbstate_t mbstate;
+  size_t k, n;
+
+  if (!s || !*s)
+    return 0;
+
+  n = mutt_strlen (s);
+  memset (&mbstate, 0, sizeof (mbstate));
+  k = mbrtowc (&wc, s, n, &mbstate);
+  if (width)
+    *width = wcwidth (wc);
+  return (k == (size_t)(-1) || k == (size_t)(-2)) ? -1 : k;
+}
+
 /*
  * mutt_strwidth is like mutt_strlen except that it returns the width
  * refering to the number of characters cells.
