@@ -470,7 +470,6 @@ mutt_hcache_per_folder(const char *path, const char *folder,
 {
   static char hcpath[_POSIX_PATH_MAX];
   struct stat sb;
-  MD5_CTX md5;
   unsigned char md5sum[16];
   char* s;
   int ret, plen;
@@ -495,9 +494,7 @@ mutt_hcache_per_folder(const char *path, const char *folder,
   }
   else
   {
-    MD5Init(&md5);
-    MD5Update(&md5, (unsigned char *) folder, strlen(folder));
-    MD5Final(md5sum, &md5);
+    md5_buffer (folder, strlen (folder), &md5sum);
 
     ret = snprintf(hcpath, _POSIX_PATH_MAX,
                    "%s/%02x%02x%02x%02x%02x%02x%02x%02x"
@@ -1021,3 +1018,20 @@ mutt_hcache_open(const char *path, const char *folder, hcache_namer_t namer)
     return NULL;
   }
 }
+
+#if HAVE_DB4
+const char *mutt_hcache_backend (void)
+{
+  return DB_VERSION_STRING;
+}
+#elif HAVE_GDBM
+const char *mutt_hcache_backend (void)
+{
+  return gdbm_version;
+}
+#elif HAVE_QDBM
+const char *mutt_hcache_backend (void)
+{
+  return "qdbm " _QDBM_VERSION;
+}
+#endif
