@@ -230,7 +230,7 @@ int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes, progress_t* pbar)
     fputc (c, fp);
     
     if (pbar && !(pos % 1024))
-      mutt_progress_update (pbar, pos);
+      mutt_progress_update (pbar, pos, -1);
 #ifdef DEBUG
     if (debuglevel >= IMAP_LOG_LTRL)
       fputc (c, debugfile);
@@ -370,6 +370,7 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
     if (!imap_authenticate (idata))
     {
       idata->state = IMAP_AUTHENTICATED;
+      new = 1;
       if (idata->conn->ssf)
 	dprint (2, (debugfile, "Communication encrypted at %d bits\n",
 		    idata->conn->ssf));
@@ -1460,11 +1461,15 @@ int imap_buffy_check (int force)
      * IDLEd elsewhere */
     if (!imap_mxcmp (name, idata->mailbox))
       continue;
-      
-    if (!lastdata)
-      lastdata = idata;
 
-    if (idata != lastdata)
+    if (!mutt_bit_isset (idata->capabilities, IMAP4REV1) &&
+        !mutt_bit_isset (idata->capabilities, STATUS))
+    {
+      dprint (2, (debugfile, "Server doesn't support STATUS\n"));
+      continue;
+    }
+
+    if (lastdata && idata != lastdata)
     {
       /* Send commands to previous server. Sorting the buffy list
        * may prevent some infelicitous interleavings */
@@ -1474,13 +1479,9 @@ int imap_buffy_check (int force)
       lastdata = NULL;
     }
 
-    if (!mutt_bit_isset (idata->capabilities, IMAP4REV1) &&
-        !mutt_bit_isset (idata->capabilities, STATUS))
-    {
-      dprint (2, (debugfile, "Server doesn't support STATUS\n"));
-      continue;
-    }
-    
+    if (!lastdata)
+      lastdata = idata;
+
     imap_munge_mbox_name (munged, sizeof (munged), name);
     snprintf (command, sizeof (command),
 	      "STATUS %s (UIDNEXT UIDVALIDITY UNSEEN RECENT)", munged);
@@ -1865,7 +1866,7 @@ imap_complete_hosts (char *dest, size_t len)
   int matchlen;
   
   matchlen = mutt_strlen (dest);
-  for (mailbox = Incoming; mailbox && mailbox->next; mailbox = mailbox->next)
+  for (mailbox = Incoming; mailbox; mailbox = mailbox->next)
   {
     if (!mutt_strncmp (dest, mailbox->path, matchlen))
     {
