@@ -40,6 +40,10 @@
 #include "imap/imap.h"
 #endif
 
+#ifdef USE_HCACHE
+#include "hcache.h"
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <locale.h>
@@ -183,7 +187,11 @@ static void show_version (void)
   printf ("\nlibidn: %s (compiled with %s)", stringprep_check_version (NULL), 
 	  STRINGPREP_VERSION);
 #endif
-  
+
+#ifdef USE_HCACHE
+  printf ("\nhcache backend: %s", mutt_hcache_backend ());
+#endif
+
   puts (_("\nCompile options:"));
 
 #ifdef DOMAIN
@@ -239,12 +247,6 @@ static void show_version (void)
 #else
 	"-USE_FLOCK   "
 #endif
-	
-#ifdef USE_INODESORT
-	"+USE_INODESORT   "
-#else
-	"-USE_INODESORT   "
-#endif
 	);
   puts (
 #ifdef USE_POP
@@ -821,7 +823,15 @@ int main (int argc, char **argv)
       for (i = optind; i < argc; i++)
       {
 	if (url_check_scheme (argv[i]) == U_MAILTO)
-	  url_parse_mailto (msg->env, &bodytext, argv[i]);
+	{
+	  if (url_parse_mailto (msg->env, &bodytext, argv[i]) < 0)
+	  {
+	    if (!option (OPTNOCURSES))
+	      mutt_endwin (NULL);
+	    fputs (_("Failed to parse mailto: link\n"), stderr);
+	    exit (1);
+	  }
+	}
 	else
 	  msg->env->to = rfc822_parse_adrlist (msg->env->to, argv[i]);
       }
