@@ -43,7 +43,7 @@
 #endif
 
 #ifndef _POSIX_PATH_MAX
-#include <posix1_lim.h>
+#include <limits.h>
 #endif
 
 #include <pwd.h>
@@ -400,6 +400,9 @@ enum
   OPTMARKOLD,
   OPTMENUSCROLL,	/* scroll menu instead of implicit next-page */
   OPTMENUMOVEOFF,	/* allow menu to scroll past last entry */
+#if defined(USE_IMAP) || defined(USE_POP)
+  OPTMESSAGECACHECLEAN,
+#endif
   OPTMETAKEY,		/* interpret ALT-x as ESC-x */
   OPTMETOO,
   OPTMHPURGE,
