@@ -23,7 +23,6 @@
 #endif
 
 #include "mutt.h"
-#include "globals.h"
 #include "mutt_socket.h"
 #include "mutt_tunnel.h"
 #if defined(USE_SSL)
@@ -38,6 +37,9 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <sys/types.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #include <sys/socket.h>
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
