@@ -99,7 +99,7 @@ static int tls_socket_read (CONNECTION* conn, char* buf, size_t len)
   }
 
   ret = gnutls_record_recv (data->state, buf, len);
-  if (gnutls_error_is_fatal(ret) == 1)
+  if (ret < 0 && gnutls_error_is_fatal(ret) == 1)
   {
     mutt_error ("tls_socket_read (%s)", gnutls_strerror (ret));
     mutt_sleep (4);
@@ -121,7 +121,7 @@ static int tls_socket_write (CONNECTION* conn, const char* buf, size_t len)
   }
 
   ret = gnutls_record_send (data->state, buf, len);
-  if (gnutls_error_is_fatal(ret) == 1)
+  if (ret < 0 && gnutls_error_is_fatal(ret) == 1)
   {
     mutt_error ("tls_socket_write (%s)", gnutls_strerror (ret));
     mutt_sleep (4);
