@@ -812,7 +812,7 @@ void mutt_pretty_mailbox (char *s)
   }
 }
 
-void mutt_pretty_size (char *s, size_t len, long n)
+void mutt_pretty_size (char *s, size_t len, LOFF_T n)
 {
   if (n == 0)
     strfcpy (s, "0K", len);
@@ -821,14 +821,14 @@ void mutt_pretty_size (char *s, size_t len, long n)
   else if (n < 1023949) /* 10K - 999K */
   {
     /* 51 is magic which causes 10189/10240 to be rounded up to 10 */
-    snprintf (s, len, "%ldK", (n + 51) / 1024);
+    snprintf (s, len, OFF_T_FMT "K", (n + 51) / 1024);
   }
   else if (n < 10433332) /* 1.0M - 9.9M */
     snprintf (s, len, "%3.1fM", n / 1048576.0);
   else /* 10M+ */
   {
     /* (10433332 + 52428) / 1048576 = 10 */
-    snprintf (s, len, "%ldM", (n + 52428) / 1048576);
+    snprintf (s, len, OFF_T_FMT "M", (n + 52428) / 1048576);
   }
 }
 
@@ -1016,7 +1016,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 
     /* Do not consider filters if no pipe at end */
     n = mutt_strlen(src);
-    if (n > 0 && src[n-1] == '|')
+    if (n > 1 && src[n-1] == '|')
     {
       /* Scan backwards for backslashes */
       off = n;
@@ -1147,7 +1147,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 	cp = prefix;
 	count = 0;
 	while (count < sizeof (prefix) &&
-	       (isdigit ((unsigned char) *src) || *src == '.' || *src == '-'))
+	       (isdigit ((unsigned char) *src) || *src == '.' || *src == '-' || *src == '='))
 	{
 	  *cp++ = *src++;
 	  count++;
@@ -1195,47 +1195,83 @@ void mutt_FormatString (char *dest,		/* output buffer */
       }
 
       /* handle generic cases first */
-      if (ch == '>')
+      if (ch == '>' || ch == '*')
       {
-	/* right justify to EOL */
-	ch = *src++; /* pad char */
-	/* calculate space left on line.  if we've already written more data
-	   than will fit on the line, ignore the rest of the line */
-	count = (COLS < destlen ? COLS : destlen);
-	if (count > col)
+	/* %>X: right justify to EOL, left takes precedence
+	 * %*X: right justify to EOL, right takes precedence */
+	int soft = ch == '*';
+	int pl, pw;
+	if ((pl = mutt_charlen (src, &pw)) <= 0)
+	  pl = pw = 1;
+
+	/* see if there's room to add content, else ignore */
+	if ((col < COLS && wlen < destlen) || soft)
 	{
-	  count -= col; /* how many columns left on this line */
-	  mutt_FormatString (buf, sizeof (buf), 0, src, callback, data, flags);
+	  int pad;
+
+	  /* get contents after padding */
+	  mutt_FormatString (buf, sizeof (buf), 0, src + pl, callback, data, flags);
 	  len = mutt_strlen (buf);
 	  wid = mutt_strwidth (buf);
-	  if (count > wid)
+
+	  /* try to consume as many columns as we can, if we don't have
+	   * memory for that, use as much memory as possible */
+	  pad = (COLS - col - wid) / pw;
+	  if (pad > 0 && wlen + (pad * pl) + len > destlen)
+	    pad = ((signed)(destlen - wlen - len)) / pl;
+	  if (pad > 0)
 	  {
-	    count -= wid; /* how many chars to pad */
-	    memset (wptr, ch, count);
-	    wptr += count;
-	    wlen += count;
-	    col += count;
+	    while (pad--)
+	    {
+	      memcpy (wptr, src, pl);
+	      wptr += pl;
+	      wlen += pl;
+	      col += pw;
+	    }
+	  }
+	  else if (soft && pad < 0)
+	  {
+	    /* set wptr and wlen back just enough bytes to make sure buf
+	     * fits on screen, \0-terminate dest so mutt_wstr_trunc()
+	     * can correctly compute string's length */
+	    if (pad < -wlen)
+	      pad = -wlen;
+	    *wptr = 0;
+	    wlen = mutt_wstr_trunc (dest, wlen + pad, col + pad, &col);
+	    wptr = dest + wlen;
 	  }
 	  if (len + wlen > destlen)
-	    len = destlen - wlen;
+	    len = mutt_wstr_trunc (buf, destlen - wlen, COLS - col, NULL);
 	  memcpy (wptr, buf, len);
 	  wptr += len;
 	  wlen += len;
-	  col += mutt_strwidth (buf);
+	  col += wid;
+	  src += pl;
 	}
 	break; /* skip rest of input */
       }
       else if (ch == '|')
       {
 	/* pad to EOL */
-	ch = *src++;
-	if (destlen > COLS)
-	  destlen = COLS;
-	if (destlen > wlen)
+	int pl, pw, c;
+	if ((pl = mutt_charlen (src, &pw)) <= 0)
+	  pl = pw = 1;
+
+	/* see if there's room to add content, else ignore */
+	if (col < COLS && wlen < destlen)
 	{
-	  count = destlen - wlen;
-	  memset (wptr, ch, count);
-	  wptr += count;
+	  c = (COLS - col) / pw;
+	  if (c > 0 && wlen + (c * pl) > destlen)
+	    c = ((signed)(destlen - wlen)) / pl;
+	  while (c > 0)
+	  {
+	    memcpy (wptr, src, pl);
+	    wptr += pl;
+	    wlen += pl;
+	    col += pw;
+	    c--;
+	  }
+	  src += pl;
 	}
 	break; /* skip rest of input */
       }
@@ -1268,7 +1304,7 @@ void mutt_FormatString (char *dest,		/* output buffer */
 	}
 	
 	if ((len = mutt_strlen (buf)) + wlen > destlen)
-	  len = (destlen - wlen > 0) ? (destlen - wlen) : 0;
+	  len = mutt_wstr_trunc (buf, destlen - wlen, COLS - col, NULL);
 
 	memcpy (wptr, buf, len);
 	wptr += len;
@@ -1308,9 +1344,23 @@ void mutt_FormatString (char *dest,		/* output buffer */
     }
     else
     {
-      *wptr++ = *src++;
-      wlen++;
-      col++;
+      int tmp, w;
+      /* in case of error, simply copy byte */
+      if ((tmp = mutt_charlen (src, &w)) < 0)
+	tmp = w = 1;
+      if (tmp > 0 && wlen + tmp < destlen)
+      {
+        memcpy (wptr, src, tmp);
+        wptr += tmp;
+        src += tmp;
+        wlen += tmp;
+        col += w;
+      }
+      else
+      {
+	src += destlen - wlen;
+	wlen = destlen;
+      }
     }
   }
   *wptr = 0;
