@@ -211,13 +211,13 @@ int url_ciss_tostring (ciss_url_t* ciss, char* dest, size_t len, int flags)
 
 int url_parse_mailto (ENVELOPE *e, char **body, const char *src)
 {
-  char *t;
+  char *t, *p;
   char *tmp;
   char *headers;
   char *tag, *value;
   char scratch[HUGE_STRING];
 
-  int taglen;
+  int taglen, rc = 0;
 
   LIST *last = NULL;
   
@@ -233,9 +233,9 @@ int url_parse_mailto (ENVELOPE *e, char **body, const char *src)
   url_pct_decode (tmp);
   e->to = rfc822_parse_adrlist (e->to, tmp);
 
-  tag = headers ? strtok (headers, "&") : NULL;
+  tag = headers ? strtok_r (headers, "&", &p) : NULL;
   
-  for (; tag; tag = strtok (NULL, "&"))
+  for (; tag; tag = strtok_r (NULL, "&", &p))
   {
     if ((value = strchr (tag, '=')))
       *value++ = '\0';
@@ -250,19 +250,25 @@ int url_parse_mailto (ENVELOPE *e, char **body, const char *src)
       if (body)
 	mutt_str_replace (body, value);
     }
-    else 
+    else if ((taglen = mutt_strlen (tag)) <= sizeof (scratch) - 2)
     {
-      taglen = strlen (tag);
-      /* mutt_parse_rfc822_line makes some assumptions */
+      /* only try to parse if we can format it as header for
+       * mutt_parse_rfc822_line (tag fits in scratch) */
       snprintf (scratch, sizeof (scratch), "%s: %s", tag, value);
       scratch[taglen] = '\0';
       value = &scratch[taglen+1];
       SKIPWS (value);
       mutt_parse_rfc822_line (e, NULL, scratch, value, 1, 0, 0, &last);
     }
+    else
+    {
+      rc = -1;
+      goto out;
+    }
   }
-  
+
+out:
   FREE (&tmp);
-  return 0;
+  return rc;
 }
 
